-------------------------------------------------------------------------------
-- Lorewalkers by Lockslap, US-Bleeding Hollow
-- http://ba-guild.com
-------------------------------------------------------------------------------
Lorewalkers	= LibStub("AceAddon-3.0"):NewAddon("Lorewalkers", "AceEvent-3.0", "AceConsole-3.0")
if not TomTom then return end

-- libaries
local L					= LibStub("AceLocale-3.0"):GetLocale("Lorewalkers", false)
local Astrolabe			= DongleStub("Astrolabe-1.0")
local AceConfig			= LibStub("AceConfig-3.0")
local AceConfigDialog	= LibStub("AceConfigDialog-3.0")

-- setup some globals
Lorewalkers.name 	= GetAddOnMetadata("Lorewalkers", "Title")
Lorewalkers.version	= GetAddOnMetadata("Lorewalkers", "Version")

local curZone
local waypoints, zones, db	= {}, {}, {}
local defaults 		= {
	char = {
		enabled 	= true,
		criterias	= {}
	}
}

local exaltedString		= L["You are |cff1784d1%d/%d %s (%d%%)|r with |cff1784d1The Lorewalkers|r."]
local enabledString		= ("|cff00ff00%s|r"):format(L["Enabled"])
local disabledString	= ("|cffff0000%s|r"):format(L["Disabled"])
local repStandings 		= {
	[1]	= L["Hated"],
	[2]	= L["Hostile"],
	[3]	= L["Unfriendly"],
	[4]	= L["Neutral"],
	[5]	= L["Friendly"],
	[6]	= L["Honored"],
	[7]	= L["Revered"],
	[8]	= L["Exalted"],
}

local function ZoneName(str)
	return str:lower():gsub(" ", "")
end

local function RepStatus()
	-- id 1345 are the lorewalkers
	local _, _, standing, barMin, barMax, barValue, _, _, _, _, _, _, _ = GetFactionInfoByID(1345)
	barValue = barValue - barMin
	barMax = barMax - barMin
	return barValue, barMax, repStandings[standing], math.floor(barValue / barMax * 100)
end

local function LoadCriteria(update)
	if not update then
		-- this used only once to fill the table
		for title, id in pairs(Lorewalkers.achievements) do
			for i = 1, GetAchievementNumCriteria(id) do
				-- get the achievement criteria
				local description, _, completed, _, _, _, _, _, _, _ = GetAchievementCriteriaInfo(id, i)
				db.char.criterias[description] = {
					id			= id,
					num			= i,
					completed	= completed,
				}
			end
		end
	else
		for description, criteria in pairs(db.char.criterias) do
			if not criteria.completed then
				-- only update if it was not completed the last time it was checked
				local _, _, completed, _, _, _, _, _, _, _ = GetAchievementCriteriaInfo(criteria.id, criteria.num)
				if completed == true then
					db.char.criterias[description].completed = true
				end
			end
		end
	end
end

local function ClearWaypoints()
	if #waypoints == 0 then return end
	for _, point in pairs(waypoints) do
		TomTom:RemoveWaypoint(point)
	end	
	waypoints = {}
	TomTom:ReloadWaypoints()
end

local function GetCriteriaCoordinates(criteria)
	for zone, waypoints in pairs(Lorewalkers.waypoints) do
		for name, info in pairs(waypoints) do
			if criteria == name then
				return zone, info.x, info.y
			end
		end
	end
	return false
end

function Lorewalkers:HandleChatCommand(args)
	local command, value = self:GetArgs(args, 2)
	local iconString = "\124T%s:12\124t"
	local completedString = L["%s%s Completed on |cff1784d1%d/%d/%d|r"]
	local notCompletedString = L["%s%s |cffff0000Not Completed|r"]
	local criteriaCompleted = L["    %d. %s: |cffffff00%s|r%s |cff00ff00Completed|r"]
	local criteriaNotCompleted = L["    %d. %s: |cffffff00%s|r%s |cffff0000Not Completed|r"]
	local achievementPattern = "\124c%x+\124Hachievement:(%d-):.-\124h%[.-%]\124h\124r"
	if command == "toggle" then
		db.char.enabled = not db.char.enabled
		Lorewalkers:Print((L["Waypoints have been %s"]):format(db.char.enabled == true and enabledString or disabledString))
		if not db.char.enabled then
			ClearWaypoints()
		else
			LoadCriteria(#db.char.criterias == 0 and false or true)
			curZone = ZoneName(GetZoneText())
			Lorewalkers:UpdateWaypoints(curZone)
		end
	elseif command == "rep" or command == "reputation" then
		self:Print(exaltedString:format(RepStatus()))
	elseif command == "criterias" or command == "criteria" then
		local id = 0
		if value:match(achievementPattern) then
			id = value:match(achievementPattern)
		elseif value:find(" ") then
			-- they provided a string, it must be in quotes
			if not Lorewalkers.achievements[value] then
				self:Print((L["Achievement |cffff0000\"%s\"|r doesn't exist."]):format(value))
				return
			else
				id = Lorewalkers.achievements[value]
			end
		else
			id = tonumber(value)
		end
		local link = GetAchievementLink(id)
		local _, _, _, completed, month, day, year, _, _, icon, _, _ = GetAchievementInfo(id)
		self:Print(completed == true and completedString:format(iconString:format(icon), link, month, day, year) or notCompletedString:format(iconString:format(icon), link))
		for i = 1, GetAchievementNumCriteria(id) do
			local description, _, completed, _, _, _, _, _, _, _ = GetAchievementCriteriaInfo(id, i)
			local zone, x, y = GetCriteriaCoordinates(description)
			if zone then
				print(completed == true and criteriaCompleted:format(i, description, zone, ("(%.2f, %.2f)"):format(x, y)) or criteriaNotCompleted:format(i, description, zone, ("(%.2f, %.2f)"):format(x, y)))
			else
				print(completed == true and criteriaCompleted:format(i, description, "", "") or criteriaNotCompleted:format(i, description, "", ""))
			end
		end
	elseif command == "achieve" or command == "achievements" then
		for _, id in pairs(Lorewalkers.achievements) do
			local link = GetAchievementLink(id)
			local _, _, _, completed, month, day, year, _, _, icon, _, _ = GetAchievementInfo(id)
			self:Print(completed == true and completedString:format(iconString:format(icon), link, month, day, year) or notCompletedString:format(iconString:format(icon), link))
		end
	elseif not command or command == "" or command == "help" or command == "?" then
		self:Print((L["Lorewalkers version |cff1784d1%.2f|r by |cff00ff00Lockslap|r"]):format(self.version))
		self:Print(L["Chat Command:  |cff1784d1/lorewalkers|r or |cff1784d1/lw|r"])
		print(L["    |cff1784d1help|r or |cff1784d1?|r - Print This Help Again"])
		print(L["    |cff1784d1toggle|r - Toggle the Waypoints"])
		print(L["    |cff1784d1rep|r or |cff1784d1reputation|r - Your Reputation with the Lorewalkers"])
		print(L["    |cff1784d1criteria|r or |cff1784d1criterias|r |cff1784d1<id or name>|r - Display Achievement Criteria"])
		print(L["    |cff1784d1achievements|r or |cff1784d1achieve|r - Lorewalkers Achievement Progress"])
	end
end

function Lorewalkers:UpdateWaypoints(zone)
	local loaded, completed = 0, 0
	local completedString = "|cff00ff00(%d %s)|r"
	if self.waypoints[zone] and zones[zone] then	
		-- first clear all of our waypoints
		ClearWaypoints()
		for desc, point in pairs(self.waypoints[zone]) do
			-- get the criteria name
			local _, cName = desc:match("(.+): (.+)")
			cName = cName:gsub(" %(Entrance%)", "")	-- remove entrance so it will check for the correct waypoint
			if not db.criterias[cName].completed then	-- the criteria wasnt found, or it doesnt exist
				loaded = loaded + 1
				waypoints[#waypoints + 1] = TomTom:AddZWaypoint(6, zones[zone], point.x, point.y, point.name .. "\n" .. desc, false, true, true, false, false, true)
			else
				completed = completed + 1
			end
		end
		
		-- print completed/loaded information
		if loaded > 0 then
			self:Print((L["Loaded |cff1784d1%d|r %s waypoints for |cff1784d1%s|r"]):format(loaded, completedString:format(completed, L["completed"]), GetZoneText()))
		else
			self:Print((L["You have completed all waypoints for |cff1784d1%s|r"]):format(GetZoneText()))
		end
	end
end

function Lorewalkers:HandleEvents(event, ...)
	curZone = ZoneName(GetZoneText())
	if db.char.enabled == true then
		self:UpdateWaypoints(curZone)
	end
end
Lorewalkers:RegisterEvent("ZONE_CHANGED_NEW_AREA", "HandleEvents")

function Lorewalkers:OnInitialize()
	-- load the config
	db = LibStub("AceDB-3.0"):New("LorewalkersDB", defaults)
	-- load the achievement criteria
	LoadCriteria(#db.char.criterias == 0 and false or true)
	for id, name in pairs{ GetMapZones(6) } do
		zones[ZoneName(name)] = id
	end
	TomTom.profile.arrow.enable = true
	TomTom.profile.arrow.arrival = 5
end

function Lorewalkers:OnEnable()
	self:RegisterChatCommand("lorewalkers", "HandleChatCommand")
	self:RegisterChatCommand("lw", "HandleChatCommand")
end

function Lorewalkers:OnDisable()
	self:UnregisterChatCommand("lorewalkers")
	self:UnregisterChatCommand("lw")
end